/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;
import renderer.gui.*;

import java.io.File;
import java.awt.Color;
import java.awt.event.*;
import java.util.ArrayList;

/**

*/
@SuppressWarnings("serial")
public class InteractiveModels_R5 extends InteractiveFrame
{
   private Scene scene;
   private ArrayList<Model> modelArray = new ArrayList<>();
   private int currentModel = 1;

   /**
      This constructor instantiates the Scene object
      and initializes it with appropriate geometry.
   */
   public InteractiveModels_R5(String title, int fbWidth, int fbHeight)
   {
      super(title, fbWidth, fbHeight);

      // Create several Model objects.
      modelArray.add( new ObjSimpleModel(new File("assets/apple.obj")) );
      modelArray.add( new ObjSimpleModel(new File("assets/cow.obj")) );
      modelArray.add( new ObjSimpleModel(new File("assets/galleon.obj")) );
      modelArray.add( new ObjSimpleModel(new File("assets/teapot.obj")) );
      modelArray.add( new ObjSimpleModel(new File("assets/cessna.obj")) );
      modelArray.add( new Sphere(1.0, 30, 30) );
      modelArray.add( new Cylinder(0.5, 1.0, 20, 20) );
      modelArray.add( new Torus(0.75, 0.25, 25, 25) );
      modelArray.add( new Cube2(15, 15, 15) );
      modelArray.add( new ObjSimpleModel(new File("assets/small_rhombicosidodecahedron.obj")) );
      modelArray.add( new ObjSimpleModel(new File("assets/great_rhombicosidodecahedron.obj")) );

      // Give the models random colors.
      for (Model m : modelArray)
      {
         m.setRandomColor();
      }

      // Push the models away from where the camera is.
      for (Model m : modelArray)
      {
         for (Vertex v : m.vertexList)
         {
            v.z -= 2;
         }
      }


      // Create the Scene object that we shall render
      scene = new Scene();

      // Add two models to the Scene.
      scene.addModel(modelArray.get(1),
                     modelArray.get(modelArray.size() - 1));
   }


   // Implement part of the KeyListener interface.
   @Override public void keyTyped(KeyEvent e)
   {
      //System.out.println( e );

      char c = e.getKeyChar();
      if ('h' == c)
      {
         print_help_message();
         return;
      }
      else if ('d' == c)
      {
         Pipeline.debug = ! Pipeline.debug;
       //Clip.debug = ! Clip.debug;
       //RasterizeAntialias.debug = ! RasterizeAntialias.debug
      }
      else if ( Character.isDigit(c) )
      {
         currentModel = Integer.parseInt( Character.toString(c) );
         scene.modelList.set(0, modelArray.get(currentModel));
      }
      else if ('c' == c)
      {
         // Change the solid random color of the model.
         scene.modelList.get(0).setRandomColor();
      }
      else if ('C' == c)
      {
         // Change the random color of each vertex of the model.
         scene.modelList.get(0).setRandomVertexColors();
      }
      else if ('e' == c)
      {
         // Change the solid random color of each edge of the model.
         scene.modelList.get(0).setRandomLineSegmentColors();
      }
      else if ('E' == c)
      {
         // Change the random color of each end of each edge of the model.
         scene.modelList.get(0).setRainbowLineSegmentColors();
      }
      else if ('a' == c)
      {
         Pipeline.doAntialiasing = ! Pipeline.doAntialiasing;
      }
      else if ('p' == c)
      {
         scene.camera.perspective = ! scene.camera.perspective;
      }
      else if ('s' == c) // Scale the model 10% smaller.
      {
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.x /= 1.1;  // This does NOT work with
            v.y /= 1.1;  // perspective projection.
            v.z /= 1.1;
         }
      }
      else if ('S' == c) // Scale the model 10% larger.
      {
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.x *= 1.1;  // This does NOT work with
            v.y *= 1.1;  // perspective projection.
            v.z *= 1.1;
         }
      }
      else if ('x' == c)
      {
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.x -= 0.1;
         }
      }
      else if ('X' == c)
      {
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.x += 0.1;
         }
      }
      else if ('y' == c)
      {
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.y -= 0.1;
         }
      }
      else if ('Y' == c)
      {
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.y += 0.1;
         }
      }
      else if ('z' == c)
      {
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.z -= 0.1;
         }
      }
      else if ('Z' == c)
      {
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.z += 0.1;
         }
      }

      // Render again.
      FrameBuffer fb = this.fbp.getFrameBuffer();
      fb.clearFB(Color.black);
      Pipeline.render(scene, fb);
      fbp.update();
      repaint();
   }


   // Implement part of the ComponentListener interface.
   @Override public void componentResized(ComponentEvent e)
   {
      //System.out.println( e );

      // Get the new size of the FrameBufferPanel.
      int w = this.fbp.getWidth();
      int h = this.fbp.getHeight();

      // Create a new FrameBuffer that fits the new window size.
      FrameBuffer fb = new FrameBuffer(w, h);
      this.fbp.setFrameBuffer(fb);
      fb.clearFB(Color.black);
      Pipeline.render(scene, fb);
      fbp.update();
      repaint();
   }


   /**
      Create an instance of this class which has
      the affect of creating the GUI application.
   */
   public static void main(String[] args)
   {
      print_help_message();

      // Define initial dimensions for a FrameBuffer.
      int width  = 1024;
      int height = 1024;
      // Create an InteractiveFrame containing a FrameBuffer
      // with the given dimensions. NOTE: We need to call the
      // InteractiveModels_R5 constructor in the Java GUI Event
      // Dispatch Thread, otherwise we get a race condition
      // between the constructor (running in the main() thread)
      // and the very first ComponentEvent (running in the EDT).
      javax.swing.SwingUtilities.invokeLater(
         new Runnable() // an anonymous inner class constructor
         {
            public void run() // implement the Runnable interface
            {
               // call the constructor that builds the gui
               new InteractiveModels_R5("Renderer 5", width, height);
            }
         }
      );
   }//main()


   private static void print_help_message()
   {
      System.out.println("Use the 'd' key to toggle debugging information on and off.");
      System.out.println("Use the digit keys '0' - '9' to choose a model.");
      System.out.println("Use the 'p' key to toggle between parallel and orthographic projection.");
      System.out.println("Use the x/X, y/Y, z/Z, keys to translate the model along the x, y, z axes.");
      System.out.println("Use the s/S keys to scale the size of the model.");
      System.out.println("Use the 'c' key to change the random solid model color.");
      System.out.println("Use the 'C' key to change the random vertex colors.");
      System.out.println("Use the 'e' key to change the random solid edge colors.");
      System.out.println("Use the 'E' key to change the random edge colors.");
      System.out.println("Use the 'a' key to toggle antialiasing on and off.");
      System.out.println("Use the 'h' key to redisplay this help message.");
   }
}
