/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
  This program renders a fan of lines two times, once using the
  rasterizer without any antialiasing, and a second time using
  the raterizer that implements a simple antialiasing algorithm.
  The two renderings are displayed side-by-side in two viewports
  within the framebuffer.

  To see how the antialiasing works, look at the final rendered
  image using a pixel magnifying program.
*/
public class AntialiasingDemo
{
   private static final int size = 900;
   private static final int numberOfLines = 30;

   private static final Color backGroundColor  = Color.green;
   private static final Color foreGroundColor1 = Color.red;
   private static final Color foreGroundColor2 = Color.red; // try making this different

   public static void main(String[] args)
   {
      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Create a Model object.
      Model model = new Model();

      // Add the model to the Scene.
      scene.addModel(model);

      // Create a fan of lines, coming out of (-1, -1, -1), in
      // the square with -1 <= x <= 1, -1 <= y <= 1, and z = -1.
      model.addVertex( new Vertex(-1, -1, -1) );
      model.addColor( foreGroundColor1 );
      model.addColor( foreGroundColor2 );
      for (int i = 0; i < numberOfLines; i++)
      {
         model.addVertex( new Vertex(1, -1 + 2*(double)i/(double)numberOfLines, -1) );
         model.addLineSegment(0, 2*i+1, 0, 1);  // slope < 1
         model.addVertex( new Vertex(-1 + 2*(double)i/(double)numberOfLines, 1, -1) );
         model.addLineSegment(0, 2*i+2, 0, 1);    // slope > 1
      }
      model.addVertex( new Vertex(1, 1, -1) );
      model.addLineSegment(0, 2*numberOfLines+1, 0, 1);  // 45 degree line


      // Create a FrameBuffer to hold two viewports.
      int width  = 2*size;  // wide enough to hold two viewports
      int height = size;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Give the framebuffer a background color.
      fb.clearFB(Color.black);

      Pipeline.doAntialiasing = true;

      // Set a viewport within the framebuffer.
      fb.setViewport(0, 0, size, size); //upper-left-hand-corner, width, height
      // Give the viewport a background color.
      fb.setBgColorVP(backGroundColor);
      fb.clearVP();
      // Render our scene into the current viewport.
      Pipeline.render(scene, fb);

      Pipeline.doAntialiasing = false;

      // Set a viewport within the framebuffer.
      fb.setViewport(size, 0, size, size); //upper-left-hand-corner, width, height
      // Give the viewport a background color.
      fb.clearVP();
      // Render our scene into the current viewport.
      Pipeline.render(scene, fb);

      // Save the resulting image in a file.
      fb.dumpFB2File("AntialiasingDemo.ppm");
   }
}
