/*

*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
   This program draws a fan of lines two ways, one way
   without any antialiasing, and a second way using
   Java's antialiasing algorithm.

   To see how the antialiasing works, look at the rendered
   images using a pixel magnifying program.
*/
public class AntialiasingDemo_JavaAPI_2 extends JPanel
  implements KeyListener, MouseListener
{
   private final int SIZE = 900;
   private final int NUMBER_OF_LINES = 30;

   private boolean doAntialiasing = false;

   private final Color backGroundColor  = Color.green;
   private final Color foreGroundColor1 = Color.red;
   private final Color foreGroundColor2 = Color.red;

   public AntialiasingDemo_JavaAPI_2()
   {
      Dimension dim = new Dimension(SIZE, SIZE);
      setPreferredSize(dim);

      this.addMouseListener(this);
      this.addKeyListener(this);
      //https://stackoverflow.com/questions/11487369/jpanel-doesnt-response-to-keylistener-event
      this.setFocusable(true);
   }

   @Override
   public void paintComponent(Graphics g)
   {
      super.paintComponent(g);
      // recover Graphics2D
      Graphics2D g2 = (Graphics2D)g;

      g2.setColor(backGroundColor);
      g2.fillRect(0, 0, SIZE, SIZE);
      g2.setColor(foreGroundColor1);

      if (doAntialiasing)
      {
         // draw using anti-aliased lines
         g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
      }
      else
      {
         // draw using default lines
         g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_OFF);
      }

      for (int i = 0; i <= NUMBER_OF_LINES; i++)
      {
         g2.drawLine(0, SIZE, SIZE, SIZE - (int)(i * ((double)SIZE/(double)NUMBER_OF_LINES)) );
         g2.drawLine(0, SIZE, (int)(i * ((double)SIZE/(double)NUMBER_OF_LINES)), 0 );
      }
   }


   // Implement the KeyListener interface.
   @Override public void keyPressed(KeyEvent e){}
   @Override public void keyReleased(KeyEvent e){}
   @Override public void keyTyped(KeyEvent e)
   {
      //System.out.println( e );
      char c = e.getKeyChar();
      if ('a' == c)
      {
         doAntialiasing = ! doAntialiasing;
      }
      repaint();
   }
   // Implement the MouseListener interface.
   @Override public void mouseClicked(MouseEvent e)
   {
      doAntialiasing = ! doAntialiasing;
      repaint();
   }
   @Override public void mousePressed(MouseEvent e){}
   @Override public void mouseReleased(MouseEvent e){}
   @Override public void mouseEntered(MouseEvent e){}
   @Override public void mouseExited(MouseEvent e){}


   public static void main(String[] args)
   {
      javax.swing.SwingUtilities.invokeLater(() ->
      {
         JFrame f = new JFrame();
         f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
         f.setTitle("Renderer_4");
         f.setResizable(false);
         f.add(new AntialiasingDemo_JavaAPI_2(), BorderLayout.CENTER);
         f.pack();
         f.setLocationRelativeTo(null);
         f.setVisible(true);
      });
   }
}
