/*

*/

package renderer.scene;

/**
   A {@code LineSegment} object has two integers that
   represent the endpoints of the line segment. The
   integers are indices into the {@link Vertex} list
   of a {@link Model} object.
*/
public class LineSegment
{
   public int[] vIndex = new int[2]; // indices for this line segment's vertices
   public int[] cIndex = new int[2]; // indices for this line segment's colors

   /**
      Create a {@code LineSegment} object with two integer indices.

      @param i0  index of 1st endpoint vertex of the new {@code LineSegment}
      @param i1  index of 2nd endpoint vertex of the new {@code LineSegment}
   */
   public LineSegment(int i0, int i1)
   {
      vIndex[0] = i0;
      vIndex[1] = i1;
      cIndex[0] = 0;  // default color object
      cIndex[1] = 0;  // default color object
   }


   /**
      Create a {@code LineSegment} object with two integer indices
      for the vertices and two integer indices for the colors.

      @param i0  index of 1st endpoint vertex of the new {@code LineSegment}
      @param i1  index of 2nd endpoint vertex of the new {@code LineSegment}
      @param c0  index of 1st endpoint color of the new {@code LineSegment}
      @param c1  index of 2nd endpoint color of the new {@code LineSegment}
   */
   public LineSegment(int i0, int i1, int c0, int c1)
   {
      vIndex[0] = i0;
      vIndex[1] = i1;
      cIndex[0] = c0;
      cIndex[1] = c1;
   }


   /**
      Create a {@code LineSegment} object with the same two vertex indices
      and the same two color indices from the given {@code LineSegment} object.

      @param ls  {@code LineSegment} to make a copy of
   */
   public LineSegment(LineSegment ls) // a "copy constructor"
   {
      this(ls.vIndex[0], ls.vIndex[1], ls.cIndex[0], ls.cIndex[1]);
   }


   /**
      For debugging.

      @return {@link String} representation of this {@code LineSegment} object
   */
   @Override
   public String toString()
   {
      return "Line Segment: ([" + vIndex[0] + ", " + vIndex[1] + "], [" + cIndex[0] + ", " + cIndex[1] + "])\n";
   }
}
